@@ -1,4 +1,6 @@
 from Products.Archetypes.public import ImageField
+from Products.Archetypes.public import StringField
+from Products.Archetypes.public import StringWidget
 from archetypes.schemaextender.field import ExtensionField
 from zope.component import adapts
 from zope.component import getUtility
@@ -6,35 +8,38 @@ from zope.interface import implements
 from archetypes.schemaextender.interfaces import IOrderableSchemaExtender
 from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender
 from Products.Archetypes.public import ImageWidget
+from Products.Archetypes.public import AnnotationStorage
 from Products.CMFPlone.interfaces import IPloneSiteRoot
+from Products.ATContentTypes.configuration import zconf
 
 from zope.publisher.interfaces import IPublishTraverse
 from zope.publisher.interfaces.http import IHTTPRequest
 from ZPublisher.BaseRequest import DefaultPublishTraverse
 from ZPublisher import NotFound
+from Products.validation import V_REQUIRED
+
 
 from collective.contentleadimage.interfaces import ILeadImageable
 from collective.contentleadimage.interfaces import ILeadImageSpecific
 from collective.contentleadimage import LeadImageMessageFactory as _
 from collective.contentleadimage.config import IMAGE_FIELD_NAME
-from collective.contentleadimage.config import IMAGE_MAX_SIZE
+from collective.contentleadimage.config import IMAGE_CAPTION_FIELD_NAME
+from collective.contentleadimage.config import IMAGE_SIZES
 from collective.contentleadimage.leadimageprefs import ILeadImagePrefsForm
 
+class LeadimageCaptionField(ExtensionField, StringField):
+    """ A trivial string field """
+    
 class LeadimageImageField(ExtensionField, ImageField):
     """A Image field. """
 
-    @property
-    def max_size(self):
-        portal = getUtility(IPloneSiteRoot)
-        cli_prefs = ILeadImagePrefsForm(portal)
-        return (cli_prefs.image_maxwidth, cli_prefs.image_maxheight)
-        
-
     @property
     def sizes(self):
         portal = getUtility(IPloneSiteRoot)
         cli_prefs = ILeadImagePrefsForm(portal)
-        return {'leadimage': (cli_prefs.image_width, cli_prefs.image_height)}
+        sizes = IMAGE_SIZES.copy()
+        sizes['leadimage'] = (cli_prefs.image_width, cli_prefs.image_height)
+        return sizes
         
 class LeadImageExtender(object):
     adapts(ILeadImageable)
@@ -44,22 +49,45 @@ class LeadImageExtender(object):
 
     fields = [
         LeadimageImageField(IMAGE_FIELD_NAME,
-            widget = ImageWidget(
+          required = False,
+          storage = AnnotationStorage(migrate=True),
+          languageIndependent = True,
+          max_size = zconf.ATNewsItem.max_image_dimension,
+          validators = (('isNonEmptyFile', V_REQUIRED),
+                               ('checkNewsImageMaxSize', V_REQUIRED)),
+          widget = ImageWidget(
                          label=_(u"Lead image"),
                          description=_(u"You can upload lead image. This image "
                                        u"will be displayed above the content. "
                                        u"Uploaded image will be automatically "
                                        u"scaled to size specified in the leadimage "
                                        u"control panel."),
+                         show_content_type=False,
                  ),
         ),
-            ]
+        LeadimageCaptionField(IMAGE_CAPTION_FIELD_NAME,
+            required=False,
+            searchable=False,
+            storage = AnnotationStorage(),
+            languageIndependent = False,
+            widget = StringWidget(
+                        label=_(u"Lead image caption"),
+                        description=_(u"You may enter lead image caption text"),
+                    ),
+        ),
+        ]
 
     def __init__(self, context):
          self.context = context
 
     def getFields(self):
-         return self.fields
+        portal = getUtility(IPloneSiteRoot)
+        cli_prefs = ILeadImagePrefsForm(portal)
+        portal_type = getattr(self.context, 'portal_type', None)
+        if portal_type in cli_prefs.allowed_types:
+            return self.fields
+        else:
+            return []
          
     def getOrder(self, original):
         """
@@ -71,9 +99,12 @@ class LeadImageExtender(object):
         default = original.get('default', None)
         if default:
             desc_index = default.index('description')
-            if desc_index >= 0:
+            if desc_index >= 0 and (IMAGE_FIELD_NAME in default):
                 default.remove(IMAGE_FIELD_NAME)
                 default.insert(desc_index+1, IMAGE_FIELD_NAME)
+                if IMAGE_CAPTION_FIELD_NAME in default:
+                    default.remove(IMAGE_CAPTION_FIELD_NAME)
+                    default.insert(desc_index+2, IMAGE_CAPTION_FIELD_NAME)
         return original
 
 class LeadImageTraverse(DefaultPublishTraverse):
