@@ -1,10 +1,8 @@
-from zope.interface import Interface, implements, Attribute
+from zope.interface import Interface, implements
 from zope.component import adapts
 from zope.component import getUtility
-from zope.interface import classImplements
-from zope.interface import implementedBy
+from zope.component import getMultiAdapter
 
-from zope.app.schema.vocabulary import IVocabularyFactory
 from zope.formlib import form
 from zope import schema
 from plone.app.viewletmanager.interfaces import IViewletSettingsStorage
@@ -13,47 +11,55 @@ from Products.CMFPlone.interfaces import IPloneSiteRoot
 from Products.CMFCore.interfaces import IPropertiesTool
 from Products.CMFCore.utils import getToolByName
 from plone.app.controlpanel.form import ControlPanelForm
-from Products.CMFCore.interfaces._tools import ITypesTool
+from plone.app.form.validators import null_validator
 
-from interfaces import ILeadImageable
+from Products.statusmessages.interfaces import IStatusMessage
+from plone.protect import CheckAuthenticator
+from zope.event import notify
+from plone.app.controlpanel.events import ConfigurationChangedEvent
+
+from Products.CMFPlone import PloneMessageFactory as _p
 from collective.contentleadimage import LeadImageMessageFactory as _
+from collective.contentleadimage.interfaces import ILeadImageable
+from collective.contentleadimage import config
+from ZODB.POSException import ConflictError
+
 
 class ILeadImagePrefsForm(Interface):    
     """ The view for LeadImage  prefs form. """
 
-    # remove and uncomment schema.Tuple below as soon as setting is working
-    allowed_types = Attribute("Types")
-
-    """
     allowed_types = schema.Tuple(title=_(u'Portal types'),
-                          description=_(u'Portal types allowed to store lead image'),
+                          description=_(u'Portal types lead image may be attached to.'),
                           missing_value=tuple(),
                           value_type=schema.Choice(
                                    vocabulary="plone.app.vocabularies.UserFriendlyTypes"),
                           required=False)
-    """
     
     image_width = schema.Int(title=_(u'Width'),
-                       description=_(u'Lead image width'),
+                       description=_(u'Lead image scale width. Value specified in this field '
+                                     u"is used for generating 'leadimage' scale"),
                        default=67,
                        required=True)
 
     image_height = schema.Int(title=_(u'Height'),
-                       description=_(u'Lead image height'),
+                       description=_(u'Lead image scale height. Value specified in this field '
+                                     u"is used for generating 'leadimage' scale"),
                        default=81,
                        required=True)
 
-    image_maxwidth = schema.Int(title=_(u'Maximum width'),
-                       description=_(u'Image maximum width. All uploaded images '
-                                     u'will be scaled to this width/height'),
-                       default=640,
-                       required=True)
+    desc_scale_name = schema.Choice(title=_(u"'Description' image scale"),
+                               description=_(u'Please select scale which will be used next to Description field.'),
+                               required=True,
+                               default='thumb',
+                               vocabulary = u"collective.contentleadimage.scales_vocabulary",
+                        )
 
-    image_maxheight = schema.Int(title=_(u'Maximum height'),
-                       description=_(u'Image maximum height. All uploaded images '
-                                     u'will be scaled to this width/height'),
-                       default=480,
-                       required=True)
+    body_scale_name = schema.Choice(title=_(u"'Body' image scale"),
+                               description=_(u'Please select scale which will be used in the body.'),
+                               required=True,
+                               default='mini',
+                               vocabulary = u"collective.contentleadimage.scales_vocabulary",
+                        )
 
     viewlet_description = schema.Bool(title=_(u'Show image next do Description field'),
                                   default=True,
@@ -109,21 +115,6 @@ class LeadImageControlPanelAdapter(SchemaAdapterBase):
     def set_image_width(self, image_width):
         self.cli_props.image_width = image_width
     
-    def get_image_maxheight(self):
-        return self.cli_props.image_maxheight
-
-    def set_image_maxheight(self, image_maxheight):
-        self.cli_props.image_maxheight = image_maxheight
-    
-    def get_image_maxwidth(self):
-        return self.cli_props.image_maxwidth
-
-    def set_image_maxwidth(self, image_maxwidth):
-        self.cli_props.image_maxwidth = image_maxwidth
-    
-    def get_allowed_types(self):
-        return self.cli_props.allowed_types
-
     def get_viewlet_description(self):
         manager = 'plone.belowcontenttitle'
         viewlet = 'collective.contentleadimage.thumbnail'
@@ -144,42 +135,31 @@ class LeadImageControlPanelAdapter(SchemaAdapterBase):
         viewlet = 'collective.contentleadimage.full'
         self.setViewletVisibility(manager, viewlet, value)
         
-    def set_allowed_types(self, allowed_types):
-        """
-        This code does not work yet. Please help to get it working.
-        """
-        # this is ok
-        self.cli_props.allowed_types = allowed_types
-        # there should be added ILeadImageable interface to selected types
+    def get_allowed_types(self):
+        return self.cli_props.allowed_types
         
-        types_tool = getToolByName(self.context, 'portal_types')
-        atool = getToolByName(self.context, 'archetype_tool')
-        types_voc = getUtility(IVocabularyFactory, 'plone.app.vocabularies.UserFriendlyTypes')(self.context)
-        for t in types_voc:
-            key = t.value
-            fti = types_tool.getTypeInfo(key)
-            product = fti.product
-            meta_type = fti.content_meta_type
-            klass = atool.lookupType(product, meta_type)['klass']
-            if key in allowed_types:
-                # add it
-                classImplements(klass, ILeadImageable)
-            else:
-                current = implementedBy(klass)
-                all = current.interfaces()
-                if ILeadImageable in all:
-                    # remove
-                    all.remove(ILeadImageable)
-                    ifs = [i for i in all if i is not ILeadImageable]
-                    classImplements(klass, ifs)
+    def set_allowed_types(self, allowed_types):
+        self.cli_props.allowed_types = allowed_types
+        
+    def get_desc_scale_name(self):
+        return self.cli_props.desc_scale_name
+        
+    def set_desc_scale_name(self, value):
+        self.cli_props.desc_scale_name = value
+        
+    def get_body_scale_name(self):
+        return self.cli_props.body_scale_name
     
+    def set_body_scale_name(self, value):
+        self.cli_props.body_scale_name = value
+
     image_height  = property(get_image_height, set_image_height)
     image_width   = property(get_image_width, set_image_width)
-    image_maxheight  = property(get_image_maxheight, set_image_maxheight)
-    image_maxwidth   = property(get_image_maxwidth, set_image_maxwidth)
     viewlet_description  = property(get_viewlet_description, set_viewlet_description)
     viewlet_body  = property(get_viewlet_body, set_viewlet_body)
     allowed_types = property(get_allowed_types, set_allowed_types)
+    desc_scale_name = property(get_desc_scale_name, set_desc_scale_name)
+    body_scale_name = property(get_body_scale_name, set_body_scale_name)
 
     
 class LeadImagePrefsForm(ControlPanelForm):
@@ -192,3 +172,57 @@ class LeadImagePrefsForm(ControlPanelForm):
     description = _(u'Select properties for Content Lead Image')
     form_name = _(u'Content Lead Image Settings')
             
+    # handle_edit_action and handle_cancel_action are copied from 
+    # ControlPanelForm because they are overriden by my handle_scales_action
+    @form.action(_p(u'label_save', default=u'Save'), name=u'save')
+    def handle_edit_action(self, action, data):
+        CheckAuthenticator(self.request)
+        if form.applyChanges(self.context, self.form_fields, data,
+                             self.adapters):
+            self.status = _p("Changes saved.")
+            notify(ConfigurationChangedEvent(self, data))
+            self._on_save(data)
+        else:
+            self.status = _p("No changes made.")
+
+    @form.action(_p(u'label_cancel', default=u'Cancel'),
+                 validator=null_validator,
+                 name=u'cancel')
+    def handle_cancel_action(self, action, data):
+        IStatusMessage(self.request).addStatusMessage(_p("Changes canceled."),
+                                                      type="info")
+        url = getMultiAdapter((self.context, self.request),
+                              name='absolute_url')()
+        self.request.response.redirect(url + '/plone_control_panel')
+        return ''
+            
+    @form.action(_(u'label_recreate_scales', default=u'Recreate scales'), name=u'scales', validator=null_validator)
+    def handle_scales_action(self, action, data):
+        CheckAuthenticator(self.request)
+        number = 0
+        ctool = getToolByName(self.context, 'portal_catalog')
+        items = ctool(hasContentLeadImage=True)
+        for i in items:
+            obj = i.getObject()
+            if obj is None:
+                continue
+            if not ILeadImageable.providedBy(obj):
+                continue
+
+            try:
+                state = obj._p_changed
+            except (ConflictError, KeyboardInterrupt):
+                raise
+            except:
+                state = 0
+
+            field = obj.getField(config.IMAGE_FIELD_NAME)
+            if field is not None:
+                field.removeScales(obj)
+                field.createScales(obj)
+                number = number + 1
+
+            if state is None:
+                obj._p_deactivate()
+
+        self.status = _(u"text_scales_recreated", default=u"${number} scales recreated.", mapping={'number':number})
